function [p,P]=PerimeterEst(I,n)
%function [p,P]=PerimeterEst(I,n)
%
% Compute the perimeter of the (possibly non-connected) object in an
%  n-level quantized pixel coverage segmented image I according to [1].
%
% Set n=1 for a binary image, n=inf for a non-quantized floating point image.
%
% [1] N. Sladoje and J. Lindblad. High Precision Boundary Length Estimation
% by Utilizing Gray-Level Information. IEEE Transactions on Pattern
% Analysis and Machine Intelligence, Vol. 31, No. 2, pp. 357-363, 2009. 
%
% (C) Joakim Lindblad, 2008-2010
%  Code is not optimized for speed.

if (min(I(:))<0 || max(I(:))>1)
	error('Input image must have values in the range [0,1]');
end

[l,L]=Alg1(I);
p=Gamma(n)*l;
if nargout>1
	P=Gamma(n)*L;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [l,L]=Alg1(I)
%function [l,L]=Alg1(I)
% l is estimated length (gamma=1)
% L is image with local lengths
%
% Algorithm 1, in [1]

[M,N]=size(I);
L=zeros(M,N);
for r=2:M-1
	for c=2:N-1
		tmp=Alg1_3x3(I(r-1:r+1,c-1:c+1).'); % row order
		L(r,c)=tmp;
	end
end
l=sum(L(:));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function l=Alg1_3x3(p)

if (p(7)+p(8)+p(9) < p(1)+p(2)+p(3))
	[p(1),p(7)]=deal(p(7),p(1));
	[p(2),p(8)]=deal(p(8),p(2));
	[p(3),p(9)]=deal(p(9),p(3));
end

if (p(3)+p(6)+p(9) < p(1)+p(4)+p(7))
	[p(1),p(3)]=deal(p(3),p(1));
	[p(4),p(6)]=deal(p(6),p(4));
	[p(7),p(9)]=deal(p(9),p(7));
end

if (p(4)+p(7)+p(8) < p(2)+p(3)+p(6))
	[p(2),p(4)]=deal(p(4),p(2));
	[p(3),p(7)]=deal(p(7),p(3));
	[p(6),p(8)]=deal(p(8),p(6));
end

s1=p(1)+p(4)+p(7);
s2=p(2)+p(5)+p(8);
s3=p(3)+p(6)+p(9);

ul=(s1+s2)/2;
ur=(s2+s3)/2;

if (1 <= ul) && (ul < 2-eps) %eps
	dl=s2-s1;
	ll=sqrt(1+dl.^2)/2;
else
	ll=0;
end

if (1 < ur-eps) && (ur <= 2) %eps
	dr=s3-s2;
	lr=sqrt(1+dr.^2)/2;
else
	lr=0;
end

l=ll+lr;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function g=Gamma(n,q)
%function g=Gamma(n,q=3)

if nargin<2
	q=nan;
end

if (numel(q)==1) q=repmat(q,size(n)); end

q(isnan(q) & n<=2)=1;
q(isnan(q) & n<=8)=2;
q(isnan(q) & n>8)=3;

g=2*q./(q+sqrt( (sqrt(n.^2 + q.^2) -n ).^2+q.^2));
g(n==inf)=1;
g(q==0)=1;
