function measurements = contactAnalysis(V, fibres, fibre)
% Fibre: number of fibre

% Fibre: the label closest to the middle
% Contact: If the label stretches to the boundary, else: not

% Settings
threshold = 105; % Above is fibre, below, air
r = 20; % Radius of the window, in pixel units
video = 0; % 
recordFibre = 0; % Record movies (for verification)

txx = fibres{fibre}.xx; tyy = fibres{fibre}.yy; tzz = fibres{fibre}.zz;
[T,N,B]=frenet(txx,tyy,tzz);

if recordFibre
    filename = sprintf('fibre%d_contact.avi', fibre);
    writerObj = VideoWriter(filename)
    writerObj.FrameRate = 20;
    open(writerObj);
end

if video
    figure(7), clf, axis image
    pl = subplot('Position', [0,0,1,1]);
    imagesc(ones(2*r+1,2*r+1)); % To get the right size
    im = get(gca, 'Children');
    colormap gray
    hold on
    plot(r+1,r+1, 'rx', 'MarkerSize', 20)
    set(gca, 'clim', threshold+[-5, 5])
    textbox = text(20,5, '..', 'Color', 'k', 'FontSize', 20, 'BackgroundColor',[.7 .9 .7], 'Margin', 5);
    innerRing = plot(0,0, 'r', 'LineWidth', 3);
    outerRing = plot(0,0, 'r', 'LineWidth', 3);
end

[u,v]=meshgrid(-r:r,-r:r);
 R = (u.^2+v.^2).^(1/2);
 
for kk=1:numel(txx)
    
    % The frenet vectors
    t = T(kk,:);
    n = N(kk,:);
    b = B(kk,:);
    
    % The coordinates for the interpolation plane
    PX = txx(kk)+u*n(1)+v*b(1); 
    PY = tyy(kk)+u*n(2)+v*b(2);
    PZ = tzz(kk)+u*n(3)+v*b(3);
    
    % Interpolate the plane
    Skk= trilinear8(V, PY, PZ, PX ); Skk=double(Skk);
    
    % Threshold and label components
    L=Skk; L(Skk<threshold)=0; L(L>0)=1;
    [LAB, nlab] = bwlabel(L); % Labelled    
    
    % Figure out which label that corresponds to the fibre
    maxval = 0; maxlab = -1;
    for ll = 1:nlab
        lv = max(max((100-R).*(LAB==ll)));
        if lv > maxval
            maxval=lv; maxlab=ll;
        end
    end
    
    Obj = (LAB==maxlab); % Binary image
        
    Contact(kk)=sum(sum(Obj(:,1))) + sum(sum(Obj(:,end))) + ...
        sum(sum(Obj(1,:))) + sum(sum(Obj(end,:)));
    
    allarea(kk)=sum(Obj(:));
    
    if Contact(kk)==0
       area(kk)=sum(Obj(:));
       circumference(kk)=max(1.5*pi, PerimeterEst(bwconvhull(Obj), 1));              
    end

    if video == 1
        if Contact(kk)==0
            r1 = circumference(kk)/(2*pi)
            r0 = ((r1.^2*pi-area(kk))/pi).^(1/2)
            theta = linspace(0,2*pi);        
            set(innerRing, 'Xdata', r0*cos(theta)+r+1, 'Ydata', r0*sin(theta)+r+1);
            set(outerRing, 'Xdata', r1*cos(theta)+r+1, 'Ydata', r1*sin(theta)+r+1);        
            %pause
        else
            set(innerRing, 'Xdata', r+1, 'Ydata', r+1);
            set(outerRing, 'Xdata', r+1, 'Ydata', r+1);        
        end
    end

    
    if video
        set(im, 'CData', Skk)        
        if(Contact(kk)==0)
            set(textbox, 'String', 'Free ', 'BackgroundColor',[.0 .9 .0]);
        else
             set(textbox, 'String', 'Bonded', 'BackgroundColor',[.7 .9 .7]);
        end
        if Contact(kk)==0
            pause
        end
    end
	
    %drawnow
    if recordFibre
        writeVideo(writerObj,getframe);    
    else
        drawnow
    end
            
end

if recordFibre
    close(writerObj);
end

%figure(11), 
%plot(Contact>0)
%title('Contact or free')

% Contact regions
[CL, ncl] = bwlabeln(Contact >0 );
measurements.nContactPoints = ncl;

% Free air regions
[CL, ncl] = bwlabeln(Contact == 0 );
measurements.freeLength = [];
for kk=1:ncl    
    measurements.freeLength(kk)=max(find(CL==kk))-min(find(CL==kk))+0.5;
end

if exist('circumference', 'var')
    % The fibre has to be free somewhere for this to work
    circumference = circumference(Contact==0);
    area = area(Contact==0);

    r1 = circumference/(2*pi);
    r0 = ((r1.^2*pi-area)/pi).^(1/2);

    ravg = (r0+r1)/2;
    tavg = (r1-r0);
    measurements.radius=median(ravg);
    measurements.thickness=median(tavg);
else
    measurements.radius=-1;
    measurements.thickness=-1;
end

if 0
figure, 
plot(r0, 'k', 'LineWidth', 2), 
hold on, 
plot(r1, '--r', 'LineWidth', 3)
legend({'r0 - inner', 'r1 - outer'});
end

measurememnts.area = area;
measurements.circumference = circumference;

figure,
plot(allarea, 'k')
xlabel('position')
ylabel('area')
hold on
for kk=1:numel(Contact)
    if Contact(kk) ==0
        plot(kk, allarea(kk), 'r.')
    end
end
matlab2tikz('allarea1.tex', 'height', '\figureheight', 'width', '\figurewidth');

end


