function area = bondStrength(c1, d1, c2, d2, V)
% c1 and c2: two curves throught fibres that meet
% d1 and d2: shortest distance to the other fibre i.e.
% d1(1) is the distance from c1(1,:) to any point at c2
%
% V: the volumetric image, in [0,1]. will be thresholded at 0.5
%
% c1 = [ x1 y1 z1 
%        x2 y2 z2 
%        ...
% preparation:
%  mex -largeArrayDims fiberCut3.cpp

verbose = 0;
saveBonds =1;

% coordinate change due to previous design. 
c1 = c1(:,[3 1 2]);
c2 = c2(:,[3 1 2]);

% radius of the seeed regions
r1 = 1/3*d1; 
r2 = 1/3*d2;


if verbose>1
%figure,
%plot3(c1(:,2), c1(:,1), c1(:,3));
tubeplot(c1(:,2), c1(:,1), c1(:,3), r1)
hold on
%plot3(c2(:,2), c2(:,1), c2(:,3));
tubeplot(c2(:,2), c2(:,1), c2(:,3), r2)
disp('Supplied curves')
pause
end

%% Region of interest in the volume
% Find the minimal box that contains the curves
% x
minx=min([c1(:,1); c2(:,1)]);
maxx=max([c1(:,1); c2(:,1)]);
% y
miny=min([c1(:,2); c2(:,2)]);
maxy=max([c1(:,2); c2(:,2)]);
% z
minz=min([c1(:,3); c2(:,3)]);
maxz=max([c1(:,3); c2(:,3)]);
% rounding
minx=round(minx); maxx=round(maxx);
miny=round(miny); maxy=round(maxy);
minz=round(minz); maxz=round(maxz);
% cut out the ROI
ROI = V(minx:maxx, miny:maxy, minz:maxz);
% Threshold it
ROI(ROI<100)=0;
ROI(ROI>0)=1;
ROI = uint8(ROI);
% translate the curves to match the coordinate system of ROI
rc1 = [c1(:,1)-minx+1, c1(:,2)-miny+1, c1(:,3)-minz+1];
rc2 = [c2(:,1)-minx+1, c2(:,2)-miny+1, c2(:,3)-minz+1];

ROI(:,:,1)=0; ROI(:,:,end)=0;
ROI(:,1,:)=0; ROI(:,end,:)=0;
ROI(1,:,:)=0; ROI(end,:,:)=0;

if verbose
    figure
[f,v]=isosurface(ROI,0.5);
patch('Faces', f, 'Vertices', v, 'Facecolor', [.5,.5,.5], 'EdgeAlpha', 0, 'FaceAlpha', 1);
disp('Supplied fibres')
view(3)
camlight
axis equal
pause
end

%% Use the supplied curves to add seed regions

%  Place spheres at the first curve
for kk=1:size(rc1,1)
    ROI = addball3(ROI, rc1(kk,1), rc1(kk,2), rc1(kk,3), r1(kk), 2); 
end
% At the second
for kk=1:size(rc2,1)    
    ROI = addball3(ROI, rc2(kk,1), rc2(kk,2), rc2(kk,3), r2(kk), 3); 
end

ROI(:,:,1)=0; ROI(:,:,end)=0;
ROI(:,1,:)=0; ROI(:,end,:)=0;
ROI(1,:,:)=0; ROI(end,:,:)=0;


if verbose
figure
[f,v]=isosurface(ROI==2,0.5);
patch('Faces', f, 'Vertices', v, 'Facecolor', [.5,.8,.5], 'EdgeAlpha', 0);
hold on
[f,v]=isosurface(ROI==3,0.5);
patch('Faces', f, 'Vertices', v, 'Facecolor', [.8,.5,.5], 'EdgeAlpha', 0);
view(3)
camlight
axis equal
disp('Seeds');
pause
end

if verbose
    disp('cutting')
    tic
end


%[S, area] = fiberCut3(uint8(ROI)); % Discrete graph cuts
whos ROI
tic
[S, area] = fiberCut3c(double(ROI)); % Continuous flow
toc

S(ROI==0)=0;

if verbose

    toc
disp(['Flow: ' num2str(flow)]);
figure
    [f,v]=isosurface(S==2,0.5);
    patch('Faces', f, 'Vertices', v, 'Facecolor', [.5,.8,.5], 'EdgeAlpha', 0, 'FaceAlpha', 0.6);
    hold on
    [f,v]=isosurface(S==3,0.5);
    patch('Faces', f, 'Vertices', v, 'Facecolor', [.8,.5,.5], 'EdgeAlpha', 0, 'FaceAlpha', 0.6);
    view(3)
    camlight
    axis equal
    plot3(rc1(:,2), rc1(:,1), rc1(:,3), 'g')
    plot3(rc2(:,2), rc2(:,1), rc2(:,3), 'r')
    disp('Segmentation')
    pause
end

if saveBonds
f = figure('visible', 'off');
    S=S(2:end-1,2:end-1,2:end-1);    
    [f,v]=isosurface(S==2,0.5);    
    patch('Faces', f, 'Vertices', v, 'Facecolor', [.5,.8,.5], 'EdgeAlpha', 0, 'FaceAlpha', 1);
    [f,v]=isocaps(S==2,0.5);    
    patch('Faces', f, 'Vertices', v, 'Facecolor', [.2,.6,.2], 'EdgeAlpha', 0, 'FaceAlpha', 1);
    hold on
    [f,v]=isosurface(S==3,0.5);
    patch('Faces', f, 'Vertices', v, 'Facecolor', [.8,.5,.5], 'EdgeAlpha', 0, 'FaceAlpha', 1);
    [f,v]=isocaps(S==3,0.5);    
    patch('Faces', f, 'Vertices', v, 'Facecolor', [.6,.2,.2], 'EdgeAlpha', 0, 'FaceAlpha', 1);
    view(3)
    material dull
    camlight
    axis equal
    axis off
    %plot3(rc1(:,2), rc1(:,1), rc1(:,3), 'g')
    %plot3(rc2(:,2), rc2(:,1), rc2(:,3), 'r')
    set(gca, 'Position', get(gca, 'OuterPosition') - ...
    get(gca, 'TightInset') * [-1 0 1 0; 0 -1 0 1; 0 0 1 0; 0 0 0 1]);

    d=dir('bondImages/bond*.png');
    b = numel(d)+1;
    print('-dpng', sprintf('bondImages/bond%03d.png', b));
    
end
