function interLine(I)
%{  
An Interactive Line Tool

An open contours is created over an image with a simple interface:
* Left click on the image: adds a new point
* Left click and drag on a marker: moves it around
* Right click on marker: deletes it

The coordinates of the line are copied to the workspace as xx and yy
each time that a marker is added or moved

To draw a line on the image I:
> interLineMark(I)

For a demo, just run
> interLineMark

then try
>figure, plot(x,y,'.')    

20121113
- Line no longer clickable
- Uses distance to closest (straight) line, not closest points

20121108 
- First version

erikw
%}

f = gcf; % Creates a new figure if none available
set(f, 'Pointer', 'Cross'); % To show that some input is expected

if ~exist('I', 'var')
    I = imread('cameraman.tif');
end

img = imagesc(I);
colormap gray, axis image, hold on, axis off

% Settings
markerSize = 9;
interpolationType = 'cubic'; % cubic/spline/linear See interp1
nOutPoints = 500; % Number of points to output to workspace

% Internal initialisation
aH = gca;
h = 0;
Markers=[]; %stores pointers to all the markers
Line = [];
hasLine = 0;  % state variable, if there is a line or not
set(f, 'WindowButtonUpFcn', @stopDragFcn);
set(f, 'WindowButtonDownFcn', @windowClick);

    function startDragFcn(varargin)
        set(f, 'WindowButtonMotionFcn', @draggingFcn)        
    end
    
    function draggingFcn(varargin)        
        pt = get(aH, 'CurrentPoint');
        set(gco, 'Xdata', pt(1,1))
        set(gco, 'Ydata', pt(1,2))                        
        updateLine
    end

    function stopDragFcn(varargin)  
        % Stop following the mouse
        set(f, 'WindowButtonMotionFcn', '')                        
        % Export the line, but give it unit speed first
        xdata = get(Line, 'Xdata');
        ydata = get(Line, 'Ydata');
        if numel(xdata)>2 % Return data to workspace
            [xout, yout]=unitspeed2(xdata, ydata);
            assignin('base', 'xx', xout)
            assignin('base', 'yy', yout)
        end
    end

    function windowClick(varargin)        
        
        %% delete markers if they where clicked with the right button
        if numel(Markers)>0
        markerNo = find(gco==Markers);
        if numel(markerNo)>0
            if strcmp(get(gcbf, 'SelectionType'), 'alt')
                delete(gco)
                Markers = Markers([1:(markerNo-1) (markerNo+1):end]);
                updateLine                
            end
        end
        end
        
        % If the image surface was clicked, add a new point
        if gco == img 
            % i) create and show a new marker
            pt = get(aH, 'CurrentPoint'); x = pt(1,1); y = pt(1,2);
            h = plot(pt(1,1), pt(1,2), 'yo', 'MarkerSize', markerSize);        
            set(h, 'ButtonDownFcn', @startDragFcn)
            
            % ii) Figure out where it belongs among the existing points
            snapType = 2;
            %  1  Closest points -- Not always intuitive
            %  2  Closest (straight) line
            %  -  Where it causes the least change in the total line length
            %  -  Closest actual line
                                                           
            if snapType == 1 % Closest points
                % First, extract the closest and second closest point
                cmarkerDist =10^12;
                if (numel(Markers)>0)
                    for kk=1:numel(Markers)
                        mx = get(Markers(kk), 'Xdata');
                        my = get(Markers(kk), 'Ydata');
                        d = ((mx(1)-x)^2+(my(1)-y)^2)^(1/2);
                        if d<cmarkerDist
                            cmarkerDist=d;
                            cmarker = kk;
                        end
                    end
                    
                    if (numel(Markers)>1)
                        dmarkerDist =10^12;
                        for kk=[1:(cmarker-1) (cmarker+1):numel(Markers)];
                            mx = get(Markers(kk), 'Xdata');
                            my = get(Markers(kk), 'Ydata');
                            d = ((mx(1)-x)^2+(my(1)-y)^2)^(1/2);
                            if d<dmarkerDist
                                dmarkerDist=d;
                                dmarker = kk;
                            end
                        end
                    end
                    
                    % cmarker is the closest marker, dmarker the second closest
                    if exist('dmarker', 'var')
                        % There are two closest markers, but if one of them is the end
                        % point, we'd like the new marker to stick to that one only
                        
                        if cmarker == 1 && (cmarkerDist < 2*dmarkerDist)
                            Markers = [h Markers];
                        elseif cmarker == numel(Markers) && (cmarkerDist < 2*dmarkerDist)
                            Markers = [Markers h];
                        else
                            pos = min(cmarker, dmarker);
                            Markers = [Markers(1:pos) h Markers((pos+1):numel(Markers))];
                        end
                        
                    else % If
                        if cmarker == 1;
                            Markers = [h Markers];
                        else
                            Markers = [Markers h];
                        end
                    end
                else                
                    Markers = [Markers h];
                end
                updateLine;
            else
                % If something else than the image was clicked
                % Nothing, but the clicked point will be moved
            
            end % if snapType == 1
     
            if snapType == 2; % Distance to closest line used, the most intuitive that I know of                
                nMarkers = numel(Markers);
                if nMarkers == 0
                    Markers = h;
                    updateLine
                end
                
               if nMarkers == 1                   
                   Markers = [Markers h];
               end
               
               if nMarkers > 1
                   
                    minLineDist = 10^99; % Shortest distance to any line                    
                    p1Dist = 10^9; % Distance to first point
                    pendDist = 10^9; % Distance to last point
                    
                    for kk=1:(nMarkers-1) % Loop over all line segments
                        px = get(Markers(kk), 'Xdata'); py = get(Markers(kk), 'Ydata');
                        qx = get(Markers(kk+1), 'Xdata'); qy = get(Markers(kk+1), 'Ydata');                        
                        t = [qx-px, qy-py]; 
                        lineLength = norm(t);
                        t = t/lineLength; % Line tangent
                        n = [t(2), -t(1)]; % Line normal
                        lx = x - px;
                        ly = y - py;
                        tc = t(1)*lx + t(2)*ly; % Tangential coordinate
                        nc = n(1)*lx + n(2)*ly; % Normal coordinate                                                                   
                        % Distance to first point                               
                        if kk==1 && tc < 0  
                                p1Dist = (nc^2+tc^2)/4;                                
                        end
                        % Distance to last point                                                                    
                        if kk==(nMarkers-1) && (tc-lineLength)>0;  
                            pendDist = (nc^2+(lineLength-tc)^2)/4;
                        end
                        % Distance to line
                        if tc > lineLength
                            tc = tc-lineLength;                            
                        elseif tc < 0                        
                            tc = -tc;                            
                        else                            
                            tc = 0;
                        end                        
                        lineDist = tc.^2+nc.^2; % Square of distance to line
                        if lineDist < minLineDist
                            minLineDist = lineDist;
                            pointPosition = kk;                    
                        end
                    end
                                        
                    if minLineDist < min(p1Dist, pendDist)
                        Markers = [Markers(1:pointPosition) h Markers((pointPosition+1):end)];
                    else
                        if p1Dist < pendDist
                            Markers = [h Markers];
                        else
                            Markers = [Markers h];
                        end
                    end
               end
                   
               updateLine;
            end
        end
    end

    function updateLine(varargin)
        % Draws the line or nothing according to the markers

        % If no markers, delete the line
        if numel(Markers)==0
            hasLine = 0;
            delete(Line)
            Line = [];
        end
        
        % If one marker, create a line (point) if none available
        if numel(Markers)==1
            x=get(Markers(1), 'Xdata');
            y=get(Markers(1), 'Ydata');        
            if hasLine == 0;
                hasLine = 1;
                Line = plot(x,y, 'r');                 
                set(Line, 'HitTest', 'off');
                uistack(Line, 'down');                 
            else
                set(Line, 'Xdata', x)
                set(Line, 'Ydata', y)                    
            end
        end
        
        % If multiple markers, just update the line
        if(numel(Markers)>1) 
            for kk=1:numel(Markers)
                x(kk)=get(Markers(kk), 'Xdata');
                y(kk)=get(Markers(kk), 'Ydata');
            end                
            xx = interp1(1:numel(x), x, linspace(1,numel(x), 500), interpolationType);
            yy = interp1(1:numel(y), y, linspace(1,numel(y), 500), interpolationType);
            set(Line, 'Xdata', xx)
            set(Line, 'Ydata', yy)                    
        end           
    end

function [x2,y2] = unitspeed2(x,y)
% Re-interpolate the signal (x,y) so that the spacing is equal
% unitspeed(x,y,N) interpolate using N points
% Based on unitspeed2.m
    npoints = 2*numel(x); % Number of intermediate points
    xx = spline(1:numel(x),x, linspace(1,numel(x),npoints));
    yy = spline(1:numel(y),y, linspace(1,numel(y),npoints));    
    % Get the speed
    l = ((xx(2:end)-xx(1:end-1)).^2+(yy(2:end)-yy(1:end-1)).^2).^(1/2);
    l = [0, l];
    cl = cumsum(l);    
    L = cl(end); % Total length
    % Invert the speed
    icl = spline(cl, 0:numel(cl)-1, linspace(0,max(cl),nOutPoints));   % change to linear     
    % Re-interpolate with unit speed
    x2 = interp1(0:numel(xx)-1, xx, icl/max(icl)*(numel(xx)-1), interpolationType);
    y2 = interp1(0:numel(yy)-1, yy, icl/max(icl)*(numel(yy)-1), interpolationType);        
    
    
end


end

