function showz_inter(I, first, last)
% 
%    A simple interactive script to create open contours over an image.
%    Left click on the image: adds a new point
%    Left click and drag on a marker: moves it around
%    Right click on marker: deletes it
%
%    The coordinates of the line are copied to the workspace as xx and yy
%
%

f = gcf; % Creates a new figure if none available
set(f, 'Pointer', 'Cross');

if ~exist('I', 'var')
    I = imread('cameraman.tif');
end

img = imshow(I);
colormap('gray')
axis('image')
hold on
%l1 = plot([0,size(I,2)], [first,first],'k');
%set(l1, 'HitTest', 'off')
%l2 = plot([0,size(I,2)], [last,last], 'k');
%set(l2, 'HitTest', 'off')

% Settings
markerSize = 9;
markerStyle = 'ro';

% Internal
aH = gca;
h = 0;
Markers=[]; %stores pointers to all the markers

tt = 1:size(I,2);
zz = ones(size(I,2),1)*(first+(last-first)/2);
Line = plot(tt,zz, 'r');
set(Line, 'HitTest', 'off');
noLine = 1;  % state variable, if there is a line or not

set(f, 'WindowButtonUpFcn', @stopDragFcn);
set(f, 'WindowButtonDownFcn', @windowClick);

%% Create two initial markers
Markers(1)=plot(round(0.1*size(I,2)), first+ 0.5*(last-first), markerStyle, 'MarkerSize', markerSize);
Markers(2)=plot(round(0.9*size(I,2)), first+ 0.5*(last-first), markerStyle, 'MarkerSize', markerSize);
pause(0.1)
set(Markers(1), 'ButtonDownFcn', @startDragFcn)
set(Markers(2), 'ButtonDownFcn', @startDragFcn)
uistack(Markers(1), 'top');                 
uistack(Markers(2), 'top');                 

%% Create a slider
Slider = plot(size(I,2)/2*[1,1], [1, size(I,1)], 'LineWidth', 11);
set(Slider, 'ButtonDownFcn', @startSlider);

%% Display the cross section
assignin('base', 'pos', size(I,2)/2)        
assignin('base', 'zz', get(Line, 'Ydata'))        
evalin('base', 'showcross');

   function startSlider(varargin)
        set(f, 'WindowButtonMotionFcn', @slidingFcn);
    end

    function slidingFcn(varargin)
         pt = get(aH, 'CurrentPoint');         
         xpoint = max(1, pt(1,1));
         xpoint = min(size(I,2)-1, xpoint);
         
         set(gco, 'Xdata', [xpoint, xpoint]);
         assignin('base', 'pos', round(xpoint))
         evalin('base', 'showcross');
    end

    function startDragFcn(varargin)
        set(f, 'WindowButtonMotionFcn', @draggingFcn)        
    end
    
    function draggingFcn(varargin)        
        pt = get(aH, 'CurrentPoint');
        set(gco, 'Xdata', pt(1,1))
        set(gco, 'Ydata', pt(1,2))                                
        updateLine
    end

    function stopDragFcn(varargin)        
        set(f, 'WindowButtonMotionFcn', '')                
        assignin('base', 'zz', get(Line, 'Ydata'))        
        disp('exported new values for _zz_');
    end

    function windowClick(varargin)        
        
        %% delete markers if they where clicked with the right button
        if numel(Markers)>0
        markerNo = find(gco==Markers);
        if numel(markerNo)>0
            if strcmp(get(gcbf, 'SelectionType'), 'alt')
                delete(gco)
                Markers = Markers([1:(markerNo-1) (markerNo+1):end]);
                updateLine                
            end
        end
        end
        
        % If the image surface was clicked, add a new point
        if gco == img 
            pt = get(aH, 'CurrentPoint');
            % Create and show a marker
            h = plot(pt(1,1), pt(1,2), markerStyle, 'MarkerSize', markerSize);        
            set(h, 'ButtonDownFcn', @startDragFcn)
                
        % Find out where it should be added in the list
        % First, extract the closest and second closest point
        cmarkerDist =10^12;
        if (numel(Markers)>0)
            for kk=1:numel(Markers)
                mx = get(Markers(kk), 'Xdata');
                my = get(Markers(kk), 'Ydata');
                d = ((mx(1)-pt(1,1))^2+(my(1)-pt(1,2))^2)^(1/2);
                if d<cmarkerDist
                    cmarkerDist=d;
                    cmarker = kk;
                end               
            end
                        
            if (numel(Markers)>1)
                dmarkerDist =10^12;
                for kk=[1:(cmarker-1) (cmarker+1):numel(Markers)];
                    mx = get(Markers(kk), 'Xdata');
                    my = get(Markers(kk), 'Ydata');
                    d = ((mx(1)-pt(1,1))^2+(my(1)-pt(1,2))^2)^(1/2);
                    if d<dmarkerDist
                        dmarkerDist=d;
                        dmarker = kk;
                    end
                end
            end        
        
        % cmarker is the closest marker, dmarker the second closest
        if exist('dmarker', 'var')                    
            % There are two closest markers, but if one of them is the end
            % point, we'd like the new marker to stick to that one only
            
            if cmarker == 1 && (cmarkerDist < 2*dmarkerDist)
                Markers = [h Markers];        
            elseif cmarker == numel(Markers) && (cmarkerDist < 2*dmarkerDist)
                Markers = [Markers h];        
            else
                pos = min(cmarker, dmarker);                        
                Markers = [Markers(1:pos) h Markers((pos+1):numel(Markers))];            
            end
            
        else % If
            if cmarker == 1;
                Markers = [h Markers];        
            else
                Markers = [Markers h];        
            end
        end
        else
            disp('!')
            Markers = [Markers h];        
        end        
        updateLine;        
     else
         % If something else than the image was clicked
        % Nothing, but the clicked point will be moved 
     end
    end

    function updateLine(varargin)
        % There are possibly two lines to update, the one that starts at np
        % and the one that ends at np

        if numel(Markers)==0
            noLine = 1;
            delete(Line)
            Line = [];
        end
        
        if numel(Markers)==1
            x=get(Markers(1), 'Xdata');
            y=get(Markers(1), 'Ydata');        
            if noLine
                noLine = 0;
                Line = plot(x,y, 'r');                 
                uistack(Line, 'down');                 
            else
                set(Line, 'Xdata', x)
                set(Line, 'Ydata', y)                    
            end
        end
        
        if(numel(Markers)>1) % Cubic (well at lest for more points)
            for kk=1:numel(Markers)
                z(kk)=get(Markers(kk), 'Ydata');                
                t(kk)=get(Markers(kk), 'Xdata');                
            end
                
            zz = interp1(t,z, linspace(1,numel(tt), 500), 'cubic');
                        
            set(Line, 'Ydata', zz)                    
        end           
        assignin('base', 'zz', get(Line, 'Ydata'))        
        evalin('base', 'showcross');
    end
end
